@@ -21,6 +21,8 @@
 
 #include "config.h"
 
+#define USE_DBUS_COMPAT			TRUE
+
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
@@ -31,17 +33,20 @@
 #include <string.h>
 #include <time.h>
 #include <gconf/gconf-client.h>
+#ifdef USE_DBUS_COMPAT
+#include <dbus/dbus-gtype-specialized.h>
+#endif
 
 #include "gpm-info.h"
 #include "gpm-info-data.h"
 #include "gpm-common.h"
 #include "gpm-debug.h"
 #include "gpm-hal.h"
+#include "gpm-hal-power.h"
 #include "gpm-power.h"
 #include "gpm-prefs.h"
 #include "gpm-graph-widget.h"
 #include "gpm-stock-icons.h"
-#include "gpm-info-data.h"
 
 static void     gpm_info_class_init (GpmInfoClass *klass);
 static void     gpm_info_init       (GpmInfo      *info);
@@ -54,6 +59,8 @@ static void     gpm_info_finalize   (GObject      *object);
 struct GpmInfoPrivate
 {
 	GpmPower		*power;
+	GpmHal			*hal;
+	GpmHalPower		*hal_power;
 
 	GtkWidget		*rate_widget;
 	GtkWidget		*percentage_widget;
@@ -75,7 +82,150 @@ struct GpmInfoPrivate
 G_DEFINE_TYPE (GpmInfo, gpm_info, G_TYPE_OBJECT)
 
 /**
- * gpm_graph_custom_handler:
+ * gpm_info_error_quark:
+ * Return value: Our personal error quark.
+ **/
+GQuark
+gpm_info_error_quark (void)
+{
+	static GQuark quark = 0;
+	if (!quark) {
+		quark = g_quark_from_static_string ("gpm_info_error");
+	}
+	return quark;
+}
+
+/**
+ * device_list_to_strv:
+ **/
+static gchar **
+device_list_to_strv (GList *list)
+{
+	char **value = NULL;
+	char *device;
+	GList *l;
+	int i = 0;
+
+	value = g_new0 (gchar *, g_list_length (list) + 1);
+	for (l=list; l != NULL; l=l->next) {
+		device = (gchar *) l->data;
+		value[i] = g_strdup (device);
+		++i;
+	}
+	value[i] = NULL;
+	return value;
+}
+
+gboolean
+gpm_statistics_get_types (GpmInfo  *info,
+			  gchar  ***types,
+			  GError  **error)
+{
+	GList *list = NULL;
+
+	list = g_list_append (list, "charge");
+	list = g_list_append (list, "power");
+	list = g_list_append (list, "time");
+
+	*types = device_list_to_strv (list);
+
+	g_list_free (list);
+	return TRUE;
+}
+
+#define GPM_DBUS_STRUCT_INT_INT (dbus_g_type_get_struct ("GValueArray", \
+	G_TYPE_INT, G_TYPE_INT, G_TYPE_INVALID))
+#define GPM_DBUS_STRUCT_INT_INT_INT (dbus_g_type_get_struct ("GValueArray", \
+	G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_INVALID))
+
+gboolean
+gpm_statistics_get_event_log (GpmInfo    *info,
+			      gint 	  seconds,
+			      GPtrArray **array,
+			      GError    **error)
+{
+#ifdef USE_DBUS_COMPAT
+	*array = g_ptr_array_sized_new (0);
+#else	
+	GList *events, *l;
+	GpmInfoDataPoint *new;
+	GValue *value;
+
+	gpm_debug ("seconds=%i", seconds);
+	events = gpm_info_data_get_list (info->priv->events);
+	*array = g_ptr_array_sized_new (g_list_length (events));
+
+	for (l=events; l != NULL; l=l->next) {
+		new = (GpmInfoDataPoint *) l->data;
+		gpm_debug ("event log: %i: %i", info->priv->start_time + new->time, new->value);
+		value = g_new0 (GValue, 1);
+		g_value_init (value, GPM_DBUS_STRUCT_INT_INT);
+		g_value_take_boxed (value, dbus_g_type_specialized_construct (GPM_DBUS_STRUCT_INT_INT));
+		dbus_g_type_struct_set (value, 0, new->time, 1, new->value, -1);
+		g_ptr_array_add (*array, g_value_get_boxed (value));
+		g_free (value);
+	}
+#endif
+	return TRUE;
+}
+
+gboolean
+gpm_statistics_get_data (GpmInfo     *info,
+			 gint 	      seconds,
+			 const gchar *type,
+			 gint	      options,
+			 GPtrArray  **array,
+			 GError	    **error)
+{
+#ifdef USE_DBUS_COMPAT
+	*array = g_ptr_array_sized_new (0);
+#else	
+	GList *events, *l;
+	GpmInfoDataPoint *new;
+	GValue *value;
+
+	if (strcmp (type, "rate") == 0) {
+		events = gpm_info_data_get_list (info->priv->rate_data);
+	} else if (strcmp (type, "time") == 0) {
+		events = gpm_info_data_get_list (info->priv->time_data);
+	} else if (strcmp (type, "percentage") == 0) {
+		events = gpm_info_data_get_list (info->priv->percentage_data);
+	} else {
+		g_warning ("Data type %s no known!", type);
+		*error = g_error_new (gpm_info_error_quark (),
+				      GPM_INFO_ERROR_INVALID_TYPE,
+				      "Data type %s not known!", type);
+		return FALSE;
+	}
+
+	if (events == NULL) {
+		g_warning ("Data not available");
+		*error = g_error_new (gpm_info_error_quark (),
+				      GPM_INFO_ERROR_DATA_NOT_AVAILABLE,
+				      "Data not available");
+		return FALSE;
+	}		
+
+	/* TODO! */
+	gpm_debug ("seconds=%i", seconds);
+	*array = g_ptr_array_sized_new (g_list_length (events));
+
+	for (l=events; l != NULL; l=l->next) {
+		new = (GpmInfoDataPoint *) l->data;
+		gpm_debug ("data: %i: %i", new->time, new->value);
+		value = g_new0 (GValue, 1);
+		g_value_init (value, GPM_DBUS_STRUCT_INT_INT_INT);
+		g_value_take_boxed (value, dbus_g_type_specialized_construct (GPM_DBUS_STRUCT_INT_INT_INT));
+		dbus_g_type_struct_set (value, 0, new->time, 1, new->value, 2, new->colour, -1);
+		g_ptr_array_add (*array, g_value_get_boxed (value));
+		g_free (value);
+	}
+#endif
+	return TRUE;
+}
+
+/**
+ * gpm_graph_widget_custom_handler:
  * @xml: The glade file we are reading.
  * @func_name: The function name to create the object
  *
@@ -84,7 +234,7 @@ G_DEFINE_TYPE (GpmInfo, gpm_info, G_TYPE_OBJECT)
  * Return value: The custom widget.
  **/
 static GtkWidget *
-gpm_graph_custom_handler (GladeXML *xml,
+gpm_graph_widget_custom_handler (GladeXML *xml,
 			  gchar *func_name, gchar *name,
 			  gchar *string1, gchar *string2,
 			  gint int1, gint int2,
@@ -92,7 +242,7 @@ gpm_graph_custom_handler (GladeXML *xml,
 {
 	GtkWidget *widget = NULL;
 	if (strcmp ("gpm_graph_new", func_name) == 0) {
-		widget = gpm_graph_new ();
+		widget = gpm_graph_widget_new ();
 		return widget;
 	}
 	return NULL;
@@ -107,12 +257,15 @@ gpm_graph_custom_handler (GladeXML *xml,
 static void
 gpm_info_graph_update (GtkWidget *widget, GpmInfoData *data, GpmInfoData *events)
 {
+	GList *list;
 	if (! data) {
 		return;
 	}
 	if (data) {
-		gpm_graph_set_data (GPM_GRAPH (widget), data);
-		gpm_graph_set_events (GPM_GRAPH (widget), events);
+		list = gpm_info_data_get_list (data);
+		gpm_graph_widget_set_data (GPM_GRAPH_WIDGET (widget), list);
+		list = gpm_info_data_get_list (events);
+		gpm_graph_widget_set_events (GPM_GRAPH_WIDGET (widget), list);
 	} else {
 		gpm_debug ("no log data");
 	}
@@ -137,9 +290,9 @@ gpm_info_specific_device_widgets (GpmInfo *info, GpmPowerKind kind, int id)
 	GtkWidget	*widget;
 	GString		*desc;
 	GpmPowerDevice	*device = NULL;
-	char		*icon_name;
-	const char	*prefix = NULL;
-	char		 widgetname[128];
+	gchar		*icon_name;
+	const gchar	*prefix = NULL;
+	gchar		 widgetname[128];
 
 	prefix = gpm_power_kind_to_string (kind);
 
@@ -191,12 +344,12 @@ gpm_info_specific_device_widgets (GpmInfo *info, GpmPowerKind kind, int id)
 static void
 gpm_info_populate_device_information (GpmInfo *info)
 {
+	guint	   number;
+	GtkWidget *widget;
+	
 	g_return_if_fail (info != NULL);
 	g_return_if_fail (GPM_IS_INFO (info));
 
-	int	   number;
-	GtkWidget *widget;
-
 	/* Laptop battery section */
 	number = gpm_power_get_num_devices_of_kind (info->priv->power,
 						    GPM_POWER_KIND_PRIMARY);
@@ -270,18 +423,18 @@ gpm_info_create_event_viewer_tree (GtkWidget *widget)
 
 /**
  * gpm_info_get_time_string:
- * @time: A time_t value
+ * @time_secs: A time_t value
  *
  * Converts a time_t to a string description.
  * The return value must be freed using g_free().
  * Return value: The timestring, e.g. "Sat Apr 15, 15:35:40".
  **/
-static char *
-gpm_info_get_time_string (time_t time)
+static gchar *
+gpm_info_get_time_string (time_t time_secs)
 {
-	char outstr[256];
+	gchar outstr[256];
 	struct tm *tmp;
-	tmp = localtime (&time);
+	tmp = localtime (&time_secs);
 	strftime (outstr, sizeof(outstr), "%a %b %d, %T", tmp);
 	return g_strdup (outstr);
 }
@@ -295,21 +448,21 @@ gpm_info_get_time_string (time_t time)
 static void
 gpm_info_update_event_tree (GpmInfo *info)
 {
+	gchar *timestring;
+	const gchar *descstring;
+	GpmInfoDataPoint *new;
+	GtkTreeIter   iter;
+	GList *l;
+	GtkListStore *store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+	GList *events = gpm_info_data_get_list (info->priv->events);
+
 	g_return_if_fail (info != NULL);
 	g_return_if_fail (GPM_IS_INFO (info));
-	char *timestring;
-	const char *descstring;
-	GtkListStore *store;
-	GtkTreeIter   iter;
-	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
-	GpmInfoDataPoint *new;
-	GList *l;
-	GList *events = gpm_info_data_get_list (info->priv->events);
 	for (l=events; l != NULL; l=l->next) {
 		new = (GpmInfoDataPoint *) l->data;
 		timestring = gpm_info_get_time_string (info->priv->start_time + new->time);
-		descstring = gpm_graph_event_description (new->value);
+		descstring = gpm_graph_widget_event_description (new->value);
 		gpm_debug ("event log: %s: %s", timestring, descstring);
 		/* add data to the list store */
 		gtk_list_store_append (store, &iter);
@@ -469,8 +622,11 @@ gpm_info_help_cb (GtkWidget *widget,
 void
 gpm_info_show_window (GpmInfo *info)
 {
-	GtkWidget    *widget;
-	GladeXML     *glade_xml;
+	GtkWidget *widget;
+	GladeXML  *glade_xml;
+	GtkWidget *notebook;
+	int total_devices = 0;
+	int page;
 
 	g_return_if_fail (info != NULL);
 	g_return_if_fail (GPM_IS_INFO (info));
@@ -522,19 +678,19 @@ gpm_info_show_window (GpmInfo *info)
 		widget = glade_xml_get_widget (glade_xml, "graph_percentage");
 		gtk_widget_set_size_request (widget, 600, 300);
 		info->priv->percentage_widget = widget;
-		gpm_graph_set_axis_y (GPM_GRAPH (widget), GPM_GRAPH_TYPE_PERCENTAGE);
+		gpm_graph_widget_set_axis_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
 
 		widget = glade_xml_get_widget (glade_xml, "graph_rate");
 		gtk_widget_set_size_request (widget, 600, 300);
 		info->priv->rate_widget = widget;
-		gpm_graph_set_axis_y (GPM_GRAPH (widget), GPM_GRAPH_TYPE_RATE);
-		gpm_graph_enable_legend (GPM_GRAPH (widget), TRUE);
+		gpm_graph_widget_set_axis_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_RATE);
+		gpm_graph_widget_enable_legend (GPM_GRAPH_WIDGET (widget), TRUE);
 
 		widget = glade_xml_get_widget (glade_xml, "graph_time");
 		gtk_widget_set_size_request (widget, 600, 300);
 		info->priv->time_widget = widget;
-		gpm_graph_set_axis_y (GPM_GRAPH (widget), GPM_GRAPH_TYPE_TIME);
-		gpm_graph_enable_legend (GPM_GRAPH (widget), TRUE);
+		gpm_graph_widget_set_axis_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_TIME);
+		gpm_graph_widget_enable_legend (GPM_GRAPH_WIDGET (widget), TRUE);
 	}
 
 	/* set up the event viewer tree-view */
@@ -556,8 +712,7 @@ gpm_info_show_window (GpmInfo *info)
 		gtk_widget_show (info->priv->time_widget);
 		gtk_widget_show (info->priv->percentage_widget);
 	} else {
-		GtkWidget *notebook;
-		int page;
+		/* get rid of the graphs */
 		notebook = glade_xml_get_widget (info->priv->glade_xml, "notebook_main");
 		widget = glade_xml_get_widget (info->priv->glade_xml, "vbox_percentage");
 		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (widget));
@@ -570,6 +725,25 @@ gpm_info_show_window (GpmInfo *info)
 		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
 	}
 
+	/* find the total number of and type of devices */
+	total_devices += gpm_power_get_num_devices_of_kind (info->priv->power,
+							    GPM_POWER_KIND_PRIMARY);
+	total_devices += gpm_power_get_num_devices_of_kind (info->priv->power,
+							    GPM_POWER_KIND_UPS);
+	total_devices += gpm_power_get_num_devices_of_kind (info->priv->power,
+							    GPM_POWER_KIND_MOUSE);
+	total_devices += gpm_power_get_num_devices_of_kind (info->priv->power,
+							    GPM_POWER_KIND_KEYBOARD);
+
+	/* If the total number of devices is zero, hide the
+	 * 'Device Information' tab. */
+	if (total_devices == 0) {
+		notebook = glade_xml_get_widget (info->priv->glade_xml, "notebook_main");
+		widget = glade_xml_get_widget (info->priv->glade_xml, "vbox_devices");
+		page = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (widget));
+		gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
+	}
+
 	gtk_widget_show (info->priv->main_window);
 }
 
@@ -582,7 +756,9 @@ gpm_info_show_window (GpmInfo *info)
  * Adds an point to the event log
  **/
 void
-gpm_info_event_log (GpmInfo *info, GpmGraphEvent event, const char *desc)
+gpm_info_event_log (GpmInfo	       *info,
+		    GpmGraphWidgetEvent event,
+		    const gchar        *desc)
 {
 	g_return_if_fail (info != NULL);
 	g_return_if_fail (GPM_IS_INFO (info));
@@ -591,7 +767,7 @@ gpm_info_event_log (GpmInfo *info, GpmGraphEvent event, const char *desc)
 	gpm_info_data_add_always (info->priv->events,
 				  time (NULL) - info->priv->start_time,
 				  event,
-				  gpm_graph_event_colour (event),
+				  gpm_graph_widget_event_colour (event),
 				  desc);
 	if (info->priv->main_window) {
 		/* do this only if the main window is loaded */
@@ -625,11 +801,11 @@ gpm_info_log_do_poll (gpointer data)
 
 		/* set the correct colours */
 		if (battery_status.is_discharging) {
-			colour = GPM_GRAPH_COLOUR_DARK_RED;
+			colour = GPM_GRAPH_WIDGET_COLOUR_DISCHARGING;
 		} else if (battery_status.is_charging) {
-			colour = GPM_GRAPH_COLOUR_DARK_BLUE;
+			colour = GPM_GRAPH_WIDGET_COLOUR_CHARGING;
 		} else {
-			colour = GPM_GRAPH_COLOUR_DEFAULT;
+			colour = GPM_GRAPH_WIDGET_COLOUR_CHARGED;
 		}
 
 		gpm_info_data_add (info->priv->percentage_data,
@@ -677,11 +853,14 @@ gpm_info_init (GpmInfo *info)
 	/* record our start time */
 	info->priv->start_time = time (NULL);
 
+	info->priv->hal = gpm_hal_new ();
+	info->priv->hal_power = gpm_hal_power_new ();
+
 	/* set up the timer callback so we can log data */
 	g_timeout_add (GPM_INFO_DATA_POLL * 1000, gpm_info_log_do_poll, info);
 
 	/* find out if we should log and display the extra graphs */
-	info->priv->is_laptop = gpm_hal_is_laptop ();
+	info->priv->is_laptop = gpm_hal_power_is_laptop (info->priv->hal_power);
 
 	/* singleton, so okay */
 	info->priv->power = gpm_power_new ();
@@ -709,7 +888,7 @@ gpm_info_init (GpmInfo *info)
 		gpm_info_data_set_max_time (info->priv->rate_data, max_time);
 		gpm_info_data_set_max_time (info->priv->time_data, max_time);
 	}
-	glade_set_custom_handler (gpm_graph_custom_handler, info);
+	glade_set_custom_handler (gpm_graph_widget_custom_handler, info);
 }
 
 /**
@@ -737,6 +916,8 @@ gpm_info_finalize (GObject *object)
 	}
 	g_object_unref (info->priv->events);
 	g_object_unref (info->priv->power);
+	g_object_unref (info->priv->hal);
+	g_object_unref (info->priv->hal_power);
 
 	G_OBJECT_CLASS (gpm_info_parent_class)->finalize (object);
 }
