@@ -21,7 +21,9 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <glib.h>
@@ -29,6 +31,10 @@
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-help.h>
 
+#if HAVE_LIBGUNIQUEAPP
+#include <libguniqueapp/guniqueapp.h>
+#endif
+
 #include "gpm-prefs.h"
 #include "gpm-debug.h"
 #include "gpm-prefs-core.h"
@@ -65,17 +71,39 @@ gpm_prefs_close_cb (GpmPrefs *prefs)
 	exit (0);
 }
 
+#if HAVE_LIBGUNIQUEAPP
+/**
+ * guniqueapp_command_cb:
+ **/
+static void
+guniqueapp_command_cb (GUniqueApp       *app,
+		       GUniqueAppCommand command,
+		       gchar            *data,
+		       gchar            *startup_id,
+		       guint             workspace,
+		       gpointer          user_data)
+{
+	GpmPrefs *prefs = GPM_PREFS (user_data);
+	if (command == G_UNIQUE_APP_ACTIVATE) {
+		gpm_prefs_activate_window (prefs);
+	}
+}
+#endif
+
 /**
  * main:
  **/
 int
 main (int argc, char **argv)
 {
-	gboolean         verbose = FALSE;
-	GOptionContext  *context;
- 	GnomeProgram    *program;
-	GpmPrefs        *prefs = NULL;
-	GMainLoop       *loop;
+	gboolean verbose = FALSE;
+	GOptionContext *context;
+ 	GnomeProgram *program;
+	GpmPrefs *prefs = NULL;
+	GMainLoop *loop;
+#if HAVE_LIBGUNIQUEAPP
+	GUniqueApp *uniqueapp;
+#endif
 
 	const GOptionEntry options[] = {
 		{ "verbose", '\0', 0, G_OPTION_ARG_NONE, &verbose,
@@ -101,24 +129,52 @@ main (int argc, char **argv)
 
 	gpm_debug_init (verbose);
 
-	prefs = gpm_prefs_new ();
+#if HAVE_LIBGUNIQUEAPP
+	gpm_debug ("Using libguniqueapp support.");
 
-	g_signal_connect (prefs, "action-help",
-			  G_CALLBACK (gpm_prefs_help_cb), NULL);
-	g_signal_connect (prefs, "action-close",
-			  G_CALLBACK (gpm_prefs_close_cb), NULL);
+	/* Arrr! Until we depend on gtk+2 2.12 we can't just use g_unique_app_get */
+	uniqueapp = g_unique_app_get_with_startup_id ("gnome-power-preferences",
+						      g_getenv ("DESKTOP_STARTUP_ID"));
+	/* check to see if the user has another prefs window open */
+	if (g_unique_app_is_running (uniqueapp)) {
+		gpm_warning ("You have another instance running. "
+			     "This program will now close");
+		g_unique_app_activate (uniqueapp);
+		/* Until we depend on gtk+2 2.12, we need to do the startup
+		 * notification manually. Remove later... */
+		gdk_notify_startup_complete ();
+	} else {
+#else
+	gpm_warning ("No libguniqueapp support. Cannot signal other instances");
+	/* we always assume we have no other running instance */
+	if (1) {
+#endif
+		/* create a new instance of the window */
+		prefs = gpm_prefs_new ();
 
+		g_signal_connect (prefs, "action-help",
+				  G_CALLBACK (gpm_prefs_help_cb), prefs);
+		g_signal_connect (prefs, "action-close",
+				  G_CALLBACK (gpm_prefs_close_cb), prefs);
+#if HAVE_LIBGUNIQUEAPP
+		/* Listen for messages from another instances */
+		g_signal_connect (G_OBJECT (uniqueapp), "message",
+				  G_CALLBACK (guniqueapp_command_cb), prefs);
+#endif
+		loop = g_main_loop_new (NULL, FALSE);
+		g_main_loop_run (loop);
 
-	loop = g_main_loop_new (NULL, FALSE);
-
-	g_main_loop_run (loop);
-
-	g_object_unref (prefs);
+		g_object_unref (prefs);
+	}
 
 	gpm_debug_shutdown ();
 
+#if HAVE_LIBGUNIQUEAPP
+	g_object_unref (uniqueapp);
+#endif
 	g_object_unref (program);
-	g_option_context_free (context);
+/* seems to not work...
+	g_option_context_free (context); */
 
 	return 0;
 }
